"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const hash_sum_1 = __importDefault(require("hash-sum"));
const path = __importStar(require("path"));
const loaderUtils = __importStar(require("loader-utils"));
const t = __importStar(require("@babel/types"));
const parser_1 = require("@babel/parser");
const utils_1 = require("./utils");
function loader(source) {
    var _a;
    const loaderContext = this;
    (_a = loaderContext.cacheable) === null || _a === void 0 ? void 0 : _a.call(loaderContext);
    if (!(loaderContext.mode === 'development')) {
        return source;
    }
    const webpackRemainingChain = loaderUtils.getRemainingRequest(loaderContext).split('!');
    const fullPath = webpackRemainingChain[webpackRemainingChain.length - 1];
    const filename = path.relative(process.cwd(), fullPath);
    const file = parser_1.parse(source, { sourceType: 'module', plugins: ['jsx', 'typescript', 'decorators-legacy'] });
    if (!(filename.endsWith('.jsx') || filename.endsWith('.tsx'))) {
        return source;
    }
    const declaredComponents = [];
    const hotComponents = [];
    let hasDefault = false;
    for (const node of file.program.body) {
        if (t.isVariableDeclaration(node)) {
            declaredComponents.push(...utils_1.parseComponentDecls(node));
        }
        else if (t.isExportNamedDeclaration(node)) {
            const { specifiers = [], declaration } = node;
            if (t.isVariableDeclaration(declaration)) {
                hotComponents.push(...utils_1.parseComponentDecls(declaration).map(({ name }) => ({
                    local: name,
                    id: hash_sum_1.default(`${filename}-${name}`),
                })));
            }
            else if (t.isClassDeclaration(declaration)) {
                const name = declaration.id.name;
                hotComponents.push({
                    local: name,
                    id: hash_sum_1.default(`${filename}-${name}`),
                });
            }
            else if (specifiers.length) {
                for (const spec of specifiers) {
                    if (t.isExportSpecifier(spec) && t.isIdentifier(spec.exported)) {
                        if (declaredComponents.find(d => d.name === spec.local.name)) {
                            hotComponents.push({
                                local: spec.local.name,
                                id: hash_sum_1.default(`${filename}-${spec.exported.name}`)
                            });
                        }
                    }
                }
            }
        }
        else if (t.isExportDefaultDeclaration(node)) {
            const { declaration } = node;
            if (t.isIdentifier(declaration)) {
                if (declaredComponents.find(d => d.name === declaration.name)) {
                    hotComponents.push({
                        local: declaration.name,
                        id: hash_sum_1.default(`${filename}-default`)
                    });
                }
            }
            else if (utils_1.isDefineComponentCall(declaration)) {
                hotComponents.push({
                    local: '__default__',
                    id: hash_sum_1.default(`${filename}-default`)
                });
                hasDefault = true;
            }
        }
    }
    if (hotComponents.length) {
        if (hasDefault) {
            source = source.replace(/export default defineComponent/g, `const __default__ = defineComponent`) + `\nexport default __default__`;
        }
        let callbackCode = '';
        for (const { local, id } of hotComponents) {
            source +=
                `\n${local}.__hmrId = '${id}'` +
                    `\n__VUE_HMR_RUNTIME__.createRecord('${id}', ${local})`;
            callbackCode += `\n__VUE_HMR_RUNTIME__.reload("${id}", ${local})`;
        }
        source +=
            `\n/* hot reload */` +
                `\nif (module.hot) {` +
                `\n  module.hot.accept()` +
                `\n  ${callbackCode}` +
                `\n}`;
    }
    return source;
}
exports.default = loader;
;
